/* rotary.c Reads out the rotary switch. It registers the pigpio callback 
 *    which is called each time the clk gpio pi changes state.
 *    This is similar to a interrupt service routine.
 *    Copyright U. Raich Jan. 2017
 *    This program is part of a course on embedded systems held
 *    at the University of Cape Coast, Ghana in 2017
 *    It is released under the GNU Public License 
 *    For details please see https://www.gnu.org/licenses/gpl.html
 */

#include <stdio.h>
#include <stdlib.h>
#include <pigpiod_if2.h>
#include <unistd.h>
#include <stdbool.h>
#include <signal.h>
#include "rotary.h"

bool rotaryDebug = false;
int rotCounter=0;
unsigned int pbCounter=0;
int pi;

void rotarySetDebug(bool onOff) {
  rotaryDebug=onOff;
}
void pbCount (pi, gpioPin, level, tick) {
  pbCounter ++;
  printf("rotaryCount: new push button counter: %d\n",pbCounter); 
}

void rotaryCount (pi, gpioPin, level, tick) {
  int dt;
  printf("rotaryCount: level: %x tick: %d\n",level,tick);
  dt = gpio_read(pi,ROTARY_DT);
  if (dt != level)
    rotCounter ++;
  else
    rotCounter --;
  printf("rotaryCount: new rotary counter: %d\n",rotCounter); 
}

void exitProg() {
  if(rotaryDebug)
    printf("\nexitProg: Closing down and exiting\n");
  pigpio_stop(pi);
  exit(0);
}

int main(int argc, char **argv) {
  static  int retCode;
 
  rotarySetDebug(true);
  signal(SIGINT,exitProg);
  if ((retCode == pigpio_start(NULL,NULL)) < 0) {
    printf("rotary: Cannot initialize pigpio\n");
    return -1;
  }
  else
    pi=retCode;  
  if (rotaryDebug)
    printf("rotary: pigpio successfully initialized\n");
  /*
    set all pins to input */
  if ((retCode = set_mode(pi,ROTARY_SW,PI_INPUT)) < 0) {
    printf("rotary: Could not set gpio %d to input\n",ROTARY_SW);
    return -1;
  }
  if ((retCode = set_mode(pi,ROTARY_DT,PI_INPUT)) < 0) {
    printf("rotary: Could not set gpio %d to input\n",ROTARY_DT);
    return -1;
  }
  if ((retCode = set_mode(pi,ROTARY_CLK,PI_INPUT)) < 0) {
    printf("rotary: Could not set gpio %d to input\n",ROTARY_CLK);
    return -1;
  }
  /*
    switch pull_up for push button on
  */
  if ((retCode = set_pull_up_down(pi,ROTARY_SW,PI_PUD_UP)) < 0) {
    printf("rotary: Could not set pull up on pin  %d\n",ROTARY_SW);
    return -1;
  }
  /*
    register the callback for rotation
  */
  if ((retCode = callback(pi, ROTARY_CLK, EITHER_EDGE, rotaryCount)) < 0) {
    printf("rotaryCallback: Could not setup callback for rotation\n");
    return -1;
  }
  if (rotaryDebug)
    printf("rotaryCallback: Callback for rotation successfully registered\n");
  /*
    register the callback for the push button
  */
  if ((retCode = callback(pi, ROTARY_SW, FALLING_EDGE, pbCount)) < 0) {
    printf("rotaryCallback: Could not setup push button callback\n");
    return -1;
  }
  if (rotaryDebug)
    printf("rotaryCallback: Callback for push button successfully registered\n");
  
  for (;;);
  return 0;
}
